/*
  A latency statistic can be used to store the latencies in case of an remote headset.
  When calling write_to_file(...), the values submitted to the statistic are written to a csv-file.
  The created file can be found in the base directory which was specified during the creation.
  Example:

   LatencyStatistic::Ptr statistic = make_latency_statistic("latency");

   statistic->submit_capture(...);
   statistic->write_to_file();
*/

#pragma once
#include <vector>
#include <string>
#include <memory>
#include <cstdint>

#include "utility/statistic.hpp"

struct LatencyCapture
{
    uint32_t frame_id = 0;
    uint32_t frame_number = 0;
    uint32_t transform_id = 0;

    double timestamp_transform_query = 0.0;
    double timestamp_server_response = 0.0;
    double timestamp_frame_decoded = 0.0;
    double timestamp_command_construct = 0.0;
    double timestamp_command_executed = 0.0;
};

class LatencyStatistic
{
public:
    typedef std::shared_ptr<LatencyStatistic> Ptr;

public:
    LatencyStatistic(const std::string& base_directory);
 
    void interface();
    bool write_to_file();

    void submit_capture(const LatencyCapture& capture);
    void clear_captures();

private:
    bool write_captures();

    std::string build_file_name();

private:
    std::string base_directory = "";
    std::vector<LatencyCapture> captures;

    Statistic::Ptr statistic_total_latency;
    Statistic::Ptr statistic_server_response;
    Statistic::Ptr statistic_frame_decoded;
    Statistic::Ptr statistic_command_construct;
    Statistic::Ptr statistic_command_executed;
};

LatencyStatistic::Ptr make_latency_statistic(const std::string& base_directory);